/********************************************************************** 
 Freeciv - Copyright (C) 2009-2014 - Andreas Røsdal   andrearo@pvv.ntnu.no
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 ***********************************************************************/


package org.freeciv.servlet;

import java.util.regex.*;

import java.io.*;
import javax.servlet.*;
import javax.servlet.http.*;

import java.sql.*;

import javax.sql.*;
import javax.naming.*;


/**
 * This class is responsible for finding an available freeciv-web server for clients
 * based on information in the metaserver database.
 */
public class CivclientLauncher extends HttpServlet {
    private static final long serialVersionUID = 1L;
    private static String PATTERN_VALIDATE_ALPHA_NUMERIC = "[0-9a-zA-Z\\.]*";

    @SuppressWarnings("unchecked")
    public void doPost(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {

		/* Parse input parameters.. */
        String action = request.getParameter("action");
        if (action == null) action = "new";

        String civServerPort = (String)request.getParameter("civserverport");

        Connection conn = null;
        try {
            Context env = (Context) (new InitialContext().lookup("java:comp/env"));
            DataSource ds = (DataSource) env.lookup("jdbc/freeciv_mysql");
            conn = ds.getConnection();

            if (!action.equals("multi") && (action.equals("new") || action.equals("load"))) {

			  /* If user requested a new game, then get host and port for an available
               * server from the metaserver DB, and use that one. */

                String serverFetchSql = "select port from servers where state = 'Pregame' and type = 'singleplayer' and humans = '0' order by rand() limit 1";

                PreparedStatement stmt = conn.prepareStatement(serverFetchSql);
                ResultSet rs = stmt.executeQuery();
                if (rs.next()) {
                    civServerPort = Integer.toString(rs.getInt(1));
                } else {
                    response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                            "No servers available for creating a new game on.");
                }
            }

		  /* Validate port */
            PreparedStatement stmt = conn.prepareStatement("SELECT count(*) FROM servers WHERE port = ?");
            stmt.setInt(1, Integer.parseInt(civServerPort));
            ResultSet rs = stmt.executeQuery();
            rs.next();
            if (rs.getInt(1) != 1) {
                response.setHeader("result", "invalid port validation");
                response.sendError(HttpServletResponse.SC_INTERNAL_SERVER_ERROR,
                        "Invalid input values to civclient.");
                return;
            }

        } catch (Exception err) {
            response.setHeader("result", "error");
            err.printStackTrace();
            return;
        } finally {
            if (conn != null)
                try {
                    conn.close();
                } catch (SQLException e) {
                    e.printStackTrace();
                }
        }

        response.setHeader("port", civServerPort);
        response.setHeader("result", "success");
        response.setHeader("action", action);
        response.getOutputStream().print("success");

    }

    public void doGet(HttpServletRequest request, HttpServletResponse response)
            throws IOException, ServletException {
        response.getOutputStream().print("Freeciv-web civclientlauncher servlet: Please use HTTP Post.");

    }

}
